
/* 
 * SuperCollider3 source file "SFSliceInstrument.sc" 
 * Written by Jens Gulden, jgulden@cs.tu-berlin.de.
 * Licensed under the GNU General Public License (GPL),
 * this software comes with NO WARRANTY.
 */

// --- class SFSliceInstrument ------------------------------------------------
//
// <p>This class provides a bridge between XML and the SFSlice classes.</p>
//  
SFSliceInstrument {

    // --- attributes

    var <>slices; // type  Dictionary
    var <>waves; // type  Dictionary
    var <>cuelists; // type  Dictionary
    var <>tempo = 1.0; // type  Float
    var <>verbose = false; // type  boolean


    // --- new(xmlFilename) : SFSliceInstrument -------------------------------
    //      
    *new { arg xmlFilename; // type String         
        if ( xmlFilename != nil , {
            ^super.new.init.loadXML(xmlFilename); // type  SFSliceInstrument
        },{
            ^super.new.init; // type  SFSliceInstrument
        });
    } // end new        


    // --- loadXML(filename) : void -------------------------------------------
    //     
    loadXML { arg filename; // type String         
        var xml, parser, ankers, child;
        xml = DOMDocument.new(filename);
        parser = XMLParser.new;
        parser
            // <instrument>
            .register("instrument", SFSliceInstrument) // use defaultParseFunction
            // <wave name=... src=...>
            
            .register("wave", SFSlice, { arg slice, tag;
                // create masterslice representing the whole wave file
                var filename, sf;
                //"### <WAVE>".postln;
                filename = tag.getAttribute("src");
                //sf = SF3(filename, 1000);
                slice.initMaster(filename);
                slice.name_(tag.getAttribute("name"));
                this.waves.put(slice.name, slice);
                tag.getChildNodes.do({ arg ch;
                    child = parser.parse(ch, slice);
                    case
                        { child.isKindOf(SFSlice) } {
                            this.slices.put(child.name, child);
                        }
                        { child.isKindOf(SFCuelist) } {
                            this.cuelists.put(child.name, child);
                        }
                        // silently ignore unknown tags
                });
            })
            
            // <slice name=... [from=...] [to=...] [length=...]>
            .register("slice", SFSlice, { arg slice, tag, parent;
                // create slice
                var n, p, f, e, l, t, k, a, fn, sf, startAnker, endAnker, ankers;
                //"### <SLICE>".postln;
                n = tag.getAttribute("name");
                p = tag.getAttribute("parent-ref");
                f = tag.getAttribute("from");
                //e = tag.getAttributes.at("end");
                l = tag.getAttribute("length");
                t = tag.getAttribute("to");
                
                if ( p != nil , { // parent explicitly given
                    parent = this.getSlice(p);
                });
                
                // shortcut ankers
                // case 1: start and end anker given directly
                if ( f != nil , {
                    if ( l != nil , { // length-attribute
                        parent.makeSliceRangeLength(slice, n, f, l);
                    },{ // to-attribute
                        if ( t == nil , {
                            t = "1.0";
                        });
                        parent.makeSliceRange(slice, n, f, t);
                    });
                },{ // s == nil, no 'from' attribute
                    if ( l != nil , { // length attribute
                        parent.makeSlice(slice, n, l);
                    },{ // to attribute
                        if (t != nil , {
                            parent.makeSliceTo(slice, n, t); // ############### BUG HIER??
                        },{
                            "config error: tag <slice> needs at least one attribute out of 'from', 'to' or 'length'".die;
                        });
                    });
                });
                // full-sized XML ankers
                ankers = tag.getElementsByTagName("anker");
                if ( (not (ankers.isEmpty)) , {
                    ankers.do({ arg ankerTag;
                        a = parser.parse(ankerTag, slice);
                        switch ( a.name ,
                            "start", {
                                startAnker = a;
                            },
                            "end", {
                                endAnker = a;
                            });
                        slice.ankersByName.put(a.name, a);
                    });
                    slice.init(parent, n, startAnker, endAnker);
                });
            })
            
            // <cuelist> <cue slice-ref="" repeat="2"/> </cuelist>
            .register("cuelist", SFCuelist, { arg cuelist, tag, parent;
                var cueTags, sl, rep;
                //"### <CUELIST>".postln;
                cuelist.name_(tag.getAttribute("name"));
                cueTags = tag.getElementsByTagName("cue");
                cueTags.do({ arg cueTag;
                    sl = cueTag.getAttribute("slice-ref");
                    rep = cueTag.getAttribute("repeat");
                    sl = this.getSlice(sl);
                    if ( sl != nil , {
                        cuelist.masterslice_(sl.masterslice); // set multiple times, all slices MUST have the same masterslice
                        if ( ((rep == nil) || (rep == "")) , {
                            rep = "1";
                        });
                        rep = rep.asInteger;
                        cuelist.cue(sl, rep);
                    },{
                        ("config error: unknown slice name '"++sl++"'").die;
                    });
                    //slice.ankersByName.put(a.name, a);
                });
            })
            ;
            
        parser.parseObject(this, xml.getDocumentElement);
    } // end loadXML        


    // --- getSlice(name) : SFSlice -------------------------------------------
    //      
    getSlice { arg name; // type String         
        ^slices.at(name); // type  SFSlice
    } // end getSlice        


    // --- getWave(name) : SF3 ------------------------------------------------
    //      
    getWave { arg name; // type String         
        ^waves.at(name); // type  SF3
    } // end getWave        


    // --- getCuelist(name) : SFCuelist ---------------------------------------
    //      
    getCuelist { arg name; // type String         
        ^cuelists.at(name); // type  SFCuelist
    } // end getCuelist        


    // --- init() : void ------------------------------------------------------
    //   
    init {        
        this.slices = Dictionary.new;
        this.waves = Dictionary.new;
        this.cuelists = Dictionary.new;
    } // end init        


    // --- play() : void ------------------------------------------------------
    //   
    play {        
        // by default, play first cuelist
        if ( not (cuelists.isEmpty) , {
            cuelists.values.first.play(this.tempo, this.verbose);
        });
    } // end play        


} // end SFSliceInstrument
